#!/usr/bin/python3

import threading
import time
import os
import struct
import fcntl
import subprocess

# ioctl to get the device name of /dev/input/event*
EVIOCGNAME = 0x82004506

# Format for the linux /dev/input/event* events.
#   tv_sec, tv_usec, type, code, value 
EVENT_FMT = "llHHi"

EV_KEY = 0x01
KEY_LEFTCTRL = 29
KEY_LEFTSHIFT = 42
KEY_RIGHTSHIFT = 54
KEY_LEFTALT = 56
KEY_RIGHTCTRL = 97
KEY_RIGHTALT = 100
KEY_LEFTMETA = 125
KEY_RIGHTMETA = 126

class TouchpadHack:
   def __init__(self):
       self.__disabled = False
       self.__condition = threading.Condition()
       self.__disabled_until_time = time.monotonic()
       self.__running = True
       
       self.__disable_timeout = 0.5
       self.__timeout_thread = threading.Thread(target=self.timeoutThread, daemon=True)
       print("This is disable-touchpad.py for pinebook. Timeout set to", self.__disable_timeout, "seconds." )

   def findXInputDeviceNumber(self):
       res = subprocess.run(["/usr/bin/xinput", "list"], stdout=subprocess.PIPE).stdout
       print("Calling /usr/bin/xinput")
       for line in res.split(b"\n"):
           if b"pointer" in line and b"HAILUCK" in line:
               line = line[line.find(b"id=")+3:]
               line = line[:line.find(b"\t")]
               return int(line)
       return None
   
   def timeoutThread(self):
       device_number = self.findXInputDeviceNumber()
       while self.__running:
           with self.__condition:
               self.__condition.wait()
           os.system("xinput disable %d" % (device_number))
           print("xinput disable %d" % (device_number))
           while time.monotonic() < self.__disabled_until_time and self.__running:
               time.sleep(self.__disabled_until_time - time.monotonic())
           os.system("xinput enable %d" % (device_number))
           print("xinput enable %d" % (device_number))

   def inputReadThread(self):
       self.__timeout_thread.start()

       # Search for the first Pinebook keyboard entry. The first seems to be the keyboard, the 2nd the mouse.
       for event_path in sorted(os.listdir("/dev/input")):
           if not event_path.startswith("event"):
               continue
           event_path = os.path.join("/dev/input", event_path)
           f = open(event_path, "rb")
           input_device_name = bytearray(512)
           fcntl.ioctl(f, EVIOCGNAME, input_device_name, True)
           input_device_name = input_device_name[:input_device_name.find(b"\0")].decode("utf-8")
           if input_device_name == "HAILUCK CO.,LTD USB KEYBOARD":
               break
           f.close()

       try:
           while True:
               # We read events here from the keyboard and then just ignore them.
               # The only thing we care about is that an event happens.
               event = f.read(struct.calcsize(EVENT_FMT))
               tv_sec, tv_usec, etype, code, value = struct.unpack(EVENT_FMT, event)
               if etype == EV_KEY and self.isDisableKey(code):
                   self.initiateOrUpdateDisable()
       except KeyboardInterrupt:
           self.__running = False
           print("Shutting down")
       with self.__condition:
           self.__condition.notify()
       self.__timeout_thread.join() 

   def isDisableKey(self, code):
       # Don't react to modifier keys
       if code == KEY_LEFTALT or code == KEY_RIGHTALT:
           return False
       if code == KEY_LEFTCTRL or code == KEY_RIGHTCTRL:
           return False
       if code == KEY_LEFTSHIFT or code == KEY_RIGHTSHIFT:
           return False
       if code == KEY_LEFTMETA or code == KEY_RIGHTMETA:
           return False
       return True

   def initiateOrUpdateDisable(self):
       with self.__condition:
           self.__condition.notify()
           self.__disabled_until_time = time.monotonic() + self.__disable_timeout

TouchpadHack().inputReadThread()
